@@ -4,18 +4,16 @@ use crate::customer::{Address, Contact};
 use crate::error::{new_application_error, new_invalid_state_error, new_not_found_error, Error};
 use crate::internal_context::InternalContext;
 use crate::money::Currency;
-use crate::payment::handlers::step_handler;
+use crate::payment::cancel as cancel_payment;
+use crate::payment::process as process_payment;
 use crate::payment::PaymentProcessor;
-use crate::payment::{cancel as cancel_payment, Initiator, Payment};
+use crate::payment::{Initiator, Payment};
 use crate::shipping::FulfillmentTypeSelection;
 
 pub async fn get_handler<C: Context + Send>(
     ctx: &mut C,
     id: &str,
-) -> Result<
-    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    Error,
-> {
+) -> Result<Checkout<Payment<<C as PaymentProcessor>::Data>>, Error> {
     let maybe_co = ctx.get_checkout(id).await?;
 
     if maybe_co.is_none() {
@@ -35,10 +33,7 @@ pub async fn create_handler<C: Context + Send>(
     currency: Currency,
     contact: Option<Contact>,
     shipping_address: Option<Address>,
-) -> Result<
-    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    Error,
-> {
+) -> Result<Checkout<Payment<<C as PaymentProcessor>::Data>>, Error> {
     let mut co = Checkout::new(ctx, currency, contact, shipping_address).await?;
     co.populate_associations(ctx).await?;
 
@@ -50,10 +45,7 @@ pub async fn add_item_handler<C: Context + Send>(
     checkout_id: &str,
     sku: String,
     quantity: u64,
-) -> Result<
-    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    Error,
-> {
+) -> Result<Checkout<Payment<<C as PaymentProcessor>::Data>>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -74,10 +66,7 @@ pub async fn remove_item_handler<C: Context + Send>(
     checkout_id: &str,
     sku: String,
     quantity: u64,
-) -> Result<
-    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    Error,
-> {
+) -> Result<Checkout<Payment<<C as PaymentProcessor>::Data>>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -96,10 +85,7 @@ pub async fn remove_item_handler<C: Context + Send>(
 pub async fn confirm_items_handler<C: Context + Send>(
     ctx: &mut C,
     checkout_id: &str,
-) -> Result<
-    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    Error,
-> {
+) -> Result<Checkout<Payment<<C as PaymentProcessor>::Data>>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -119,10 +105,7 @@ pub async fn update_contact_handler<C: Context + Send>(
     ctx: &mut C,
     checkout_id: &str,
     contact: Contact,
-) -> Result<
-    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    Error,
-> {
+) -> Result<Checkout<Payment<<C as PaymentProcessor>::Data>>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -142,10 +125,7 @@ pub async fn update_shipping_address_handler<C: Context + Send>(
     ctx: &mut C,
     checkout_id: &str,
     shipping_address: Address,
-) -> Result<
-    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    Error,
-> {
+) -> Result<Checkout<Payment<<C as PaymentProcessor>::Data>>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -165,10 +145,7 @@ pub async fn update_fulfillment_type_handler<C: Context + Send>(
     ctx: &mut C,
     checkout_id: &str,
     fulfillment_type: FulfillmentTypeSelection,
-) -> Result<
-    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    Error,
-> {
+) -> Result<Checkout<Payment<<C as PaymentProcessor>::Data>>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -188,10 +165,7 @@ pub async fn initiate_payment_handler<C: Context + Send>(
     ctx: &mut C,
     checkout_id: &str,
     args: <C as PaymentProcessor>::ProcessArgs,
-) -> Result<
-    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    Error,
-> {
+) -> Result<Checkout<Payment<<C as PaymentProcessor>::Data>>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -212,10 +186,10 @@ pub async fn step_payment_handler<C: Context + Send>(
     ctx: &mut C,
     checkout_id: &str,
     args: <C as PaymentProcessor>::ProcessArgs,
-) -> Result<
-    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    Error,
-> {
+) -> Result<Checkout<Payment<<C as PaymentProcessor>::Data>>, Error> {
+    // first retrieve the associated payment for update to avoid deadlock
+    let maybe_payment = get_checkout_payment_for_update(ctx, checkout_id).await?;
+
     let maybe_co = ctx.get_checkout_for_update(checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -227,8 +201,23 @@ pub async fn step_payment_handler<C: Context + Send>(
 
     match co.state {
         State::PaymentInProgress(_) => {
-            let payment_id = co.payment_id.as_ref().unwrap();
-            step_handler(internal_ctx, ctx, payment_id, args).await?;
+            let mut payment = maybe_payment.ok_or(new_not_found_error(
+                "the associated payment could not be found",
+            ))?;
+
+            // check that the order payment hasn't changed
+            if &payment.id != co.payment_id.as_ref().unwrap() {
+                return Err(new_application_error("STALE_PAYMENT", "the associated payment was changed by another process while this transaction was in progress. please try again."));
+            }
+
+            process_payment(
+                internal_ctx,
+                ctx,
+                &mut payment,
+                Initiator::Checkout(&mut co),
+                args,
+            )
+            .await?;
 
             co.populate_associations(ctx).await?;
             Ok(co)
@@ -244,10 +233,7 @@ pub async fn initiate_order_handler<C: Context + Send>(
     ctx: &mut C,
     checkout_id: &str,
     args: <C as PaymentProcessor>::ProcessArgs,
-) -> Result<
-    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    Error,
-> {
+) -> Result<Checkout<Payment<<C as PaymentProcessor>::Data>>, Error> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -267,10 +253,7 @@ pub async fn release_handler<C: Context + Send>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
     checkout_id: &str,
-) -> Result<
-    Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    Error,
-> {
+) -> Result<Checkout<Payment<<C as PaymentProcessor>::Data>>, Error> {
     // first retrieve the associated payment for update to avoid deadlock
     let maybe_payment = get_checkout_payment_for_update(ctx, checkout_id).await?;
 
@@ -289,7 +272,9 @@ pub async fn release_handler<C: Context + Send>(
             co.release(ctx).await?;
         }
         State::PaymentInProgress(_) => {
-            let mut payment = maybe_payment.unwrap();
+            let mut payment = maybe_payment.ok_or(new_not_found_error(
+                "the associated payment could not be found",
+            ))?;
 
             // check that the order payment hasn't changed
             if &payment.id != co.payment_id.as_ref().unwrap() {
@@ -318,13 +303,9 @@ pub async fn release_handler<C: Context + Send>(
 async fn get_checkout_payment_for_update<C: Context + Send>(
     ctx: &mut C,
     checkout_id: &str,
-) -> Result<
-    Option<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    Error,
-> {
-    let maybe_co: Option<
-        Checkout<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
-    > = ctx.get_checkout(checkout_id).await?;
+) -> Result<Option<Payment<<C as PaymentProcessor>::Data>>, Error> {
+    let maybe_co: Option<Checkout<Payment<<C as PaymentProcessor>::Data>>> =
+        ctx.get_checkout(checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
             "the requested checkout could not be found",
@@ -332,6 +313,9 @@ async fn get_checkout_payment_for_update<C: Context + Send>(
     }
 
     let co = maybe_co.unwrap();
-    ctx.get_payment_for_update(co.payment_id.as_ref().map_or("", |payment_id| payment_id))
+    if co.payment_id.is_none() {
+        return Ok(None);
+    }
+    ctx.get_payment_for_update(co.payment_id.as_ref().unwrap())
         .await
 }
