@@ -86,45 +86,35 @@ pub trait OrderStore {
 pub trait OrderEvents: PaymentProcessor {
     async fn on_payment_in_progress(
         &mut self,
-        _order: &Order<
-            Payment<<Self as PaymentProcessor>::Data, <Self as PaymentProcessor>::ProcessArgs>,
-        >,
+        _order: &Order<Payment<<Self as PaymentProcessor>::Data>>,
     ) -> Result<(), Error> {
         Ok(())
     }
 
     async fn on_pending_confirmation(
         &mut self,
-        _order: &Order<
-            Payment<<Self as PaymentProcessor>::Data, <Self as PaymentProcessor>::ProcessArgs>,
-        >,
+        _order: &Order<Payment<<Self as PaymentProcessor>::Data>>,
     ) -> Result<(), Error> {
         Ok(())
     }
 
     async fn on_confirmed(
         &mut self,
-        _order: &Order<
-            Payment<<Self as PaymentProcessor>::Data, <Self as PaymentProcessor>::ProcessArgs>,
-        >,
+        _order: &Order<Payment<<Self as PaymentProcessor>::Data>>,
     ) -> Result<(), Error> {
         Ok(())
     }
 
     async fn on_fulfilled(
         &mut self,
-        _order: &Order<
-            Payment<<Self as PaymentProcessor>::Data, <Self as PaymentProcessor>::ProcessArgs>,
-        >,
+        _order: &Order<Payment<<Self as PaymentProcessor>::Data>>,
     ) -> Result<(), Error> {
         Ok(())
     }
 
     async fn on_cancelled(
         &mut self,
-        _order: &Order<
-            Payment<<Self as PaymentProcessor>::Data, <Self as PaymentProcessor>::ProcessArgs>,
-        >,
+        _order: &Order<Payment<<Self as PaymentProcessor>::Data>>,
     ) -> Result<(), Error> {
         Ok(())
     }
@@ -344,7 +334,7 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Order<P> {
 
 async fn reconcile<C: Context + Send>(
     ctx: &mut C,
-    order: &mut Order<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>>,
+    order: &mut Order<Payment<<C as PaymentProcessor>::Data>>,
 ) -> Result<(), Error> {
     if order.state_reconciled == true {
         return Ok(());
