@@ -1,4 +1,5 @@
 use async_trait::async_trait;
+use cloud_pubsub;
 use schemars::JsonSchema;
 use serde::de::DeserializeOwned;
 use serde::{Deserialize, Serialize};
@@ -14,9 +15,9 @@ use crate::order::Order;
 pub mod handlers;
 
 #[derive(Serialize, Deserialize, JsonSchema)]
-pub struct Payment<Data, ProcessArgs> {
+pub struct Payment<Data> {
     pub id: String,
-    pub state: PaymentState<ProcessArgs>,
+    pub state: PaymentState,
     pub initiator_id: InitiatorId,
     pub charge_amount: Money,
     pub correlation_id: Option<String>,
@@ -24,9 +25,8 @@ pub struct Payment<Data, ProcessArgs> {
 }
 
 #[derive(Serialize, Deserialize, JsonSchema)]
-pub enum PaymentState<ProcessArgs> {
-    Idle,
-    Processing(ProcessArgs),
+pub enum PaymentState {
+    Incomplete,
     Succeeded,
     Cancelled,
 }
@@ -42,6 +42,24 @@ pub enum Initiator<'a, P: Sync + Send> {
     Order(&'a mut Order<P>),
 }
 
+#[derive(Serialize, Deserialize)]
+pub struct CallbackArgs<T> {
+    pub correlation_id: String,
+    pub process_args: T,
+}
+
+impl<T: DeserializeOwned> cloud_pubsub::FromPubSubMessage for CallbackArgs<T> {
+    fn from(message: cloud_pubsub::EncodedMessage) -> Result<Self, cloud_pubsub::error::Error> {
+        match message.decode() {
+            Ok(bytes) => {
+                let val: CallbackArgs<T> = serde_json::from_slice(&bytes).unwrap();
+                Ok(val)
+            }
+            Err(e) => Err(cloud_pubsub::error::Error::from(e)),
+        }
+    }
+}
+
 #[async_trait]
 pub trait PaymentProcessor {
     type Data: Sync + Send + Serialize + DeserializeOwned;
@@ -49,7 +67,7 @@ pub trait PaymentProcessor {
 
     async fn initiate_payment(
         &mut self,
-        _payment: &Payment<Self::Data, Self::ProcessArgs>,
+        _payment: &Payment<Self::Data>,
         _args: &Self::ProcessArgs,
     ) -> Result<PaymentData<Self::Data>, Error> {
         Ok(PaymentData {
@@ -60,14 +78,14 @@ pub trait PaymentProcessor {
 
     async fn process_payment(
         &mut self,
-        payment: &Payment<Self::Data, Self::ProcessArgs>,
+        payment: &Payment<Self::Data>,
         args: &Self::ProcessArgs,
     ) -> Result<ProcessingResult<Self::Data>, Error>;
 
     // this is called synchronously and only once per release request
     async fn cancel_payment(
         &mut self,
-        payment: &Payment<Self::Data, Self::ProcessArgs>,
+        payment: &Payment<Self::Data>,
     ) -> Result<CancellationResult, Error>;
 }
 
@@ -75,18 +93,12 @@ pub trait PaymentProcessor {
 pub trait PaymentStore: PaymentProcessor {
     async fn create_payment(
         &mut self,
-        payment: &Payment<
-            <Self as PaymentProcessor>::Data,
-            <Self as PaymentProcessor>::ProcessArgs,
-        >,
+        payment: &Payment<<Self as PaymentProcessor>::Data>,
     ) -> Result<(), Error>;
 
     async fn update_payment(
         &mut self,
-        payment: &Payment<
-            <Self as PaymentProcessor>::Data,
-            <Self as PaymentProcessor>::ProcessArgs,
-        >,
+        payment: &Payment<<Self as PaymentProcessor>::Data>,
     ) -> Result<(), Error>;
 
     async fn get_payment<P: Sync + Send + Serialize + DeserializeOwned>(
@@ -110,7 +122,7 @@ pub trait PaymentStore: PaymentProcessor {
 }
 
 pub struct ProcessingResult<Data> {
-    pub state: InternalPaymentState,
+    pub state: PaymentState,
     pub payment_data: PaymentData<Data>,
 }
 
@@ -124,24 +136,16 @@ pub struct PaymentData<Data> {
     pub data: Option<Data>,
 }
 
-#[derive(Serialize, Deserialize, JsonSchema)]
-pub enum InternalPaymentState {
-    Incomplete,
-    Succeeded,
-    Failed,
-    Cancelled,
-}
-
 // constructor
 async fn new<'a, C: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
     ctx: &mut C,
     initiator: Initiator<'a, P>,
     charge_amount: Money,
     args: <C as PaymentProcessor>::ProcessArgs,
-) -> Result<Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>, Error> {
+) -> Result<Payment<<C as PaymentProcessor>::Data>, Error> {
     let mut payment = Payment {
         id: Uuid::new_v4().to_string(),
-        state: PaymentState::Idle,
+        state: PaymentState::Incomplete,
         initiator_id: match &initiator {
             Initiator::Checkout(co) => InitiatorId::Checkout(co.id.clone()),
             Initiator::Order(od) => InitiatorId::Order(od.id.clone()),
@@ -160,78 +164,41 @@ async fn new<'a, C: Context + Send, P: Sync + Send + Serialize + DeserializeOwne
     Ok(payment)
 }
 
-// public methods
-pub async fn queue_for_processing<C: Context + Send>(
-    internal_ctx: &InternalContext,
-    ctx: &mut C,
-    payment: &mut Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>,
-    args: <C as PaymentProcessor>::ProcessArgs,
-) -> Result<(), Error> {
-    match &payment.state {
-        PaymentState::Idle => {
-            payment.state = PaymentState::Processing(args);
-            ctx.update_payment(payment).await?;
-
-            internal_ctx
-                .payment_processing_topic
-                .clone()
-                .publish(&payment.id)
-                .await
-                .map_err(|_| {
-                    new_application_error(
-                        "UPSTREAM_ERROR",
-                        "couldn't send message to payment processing topic",
-                    )
-                })?;
-
-            Ok(())
-        }
-        PaymentState::Processing(_) => Err(new_application_error(
-            "PAYMENT_PROCESSING",
-            "processing is in progress",
-        )),
-        PaymentState::Succeeded | PaymentState::Cancelled => Err(new_application_error(
-            "PAYMENT_COMPLETED",
-            "the payment has already completed",
-        )),
-    }
-}
-
 pub async fn process<'a, C: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
-    payment: &mut Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>,
+    payment: &mut Payment<<C as PaymentProcessor>::Data>,
     initiator: Initiator<'a, P>,
+    args: <C as PaymentProcessor>::ProcessArgs,
 ) -> Result<(), Error> {
     match &payment.state {
-        PaymentState::Processing(args) => {
-            let result = ctx.process_payment(payment, args).await?;
-
-            payment.correlation_id = result.payment_data.correlation_id;
-            payment.data = result.payment_data.data;
-
-            match result.state {
-                InternalPaymentState::Incomplete | InternalPaymentState::Failed => {
-                    payment.state = PaymentState::Idle
-                }
-                InternalPaymentState::Succeeded => payment.state = PaymentState::Succeeded,
-                InternalPaymentState::Cancelled => payment.state = PaymentState::Cancelled,
-            }
-
-            ctx.update_payment(payment).await?;
-
-            match payment.state {
-                PaymentState::Succeeded => match initiator {
-                    Initiator::Checkout(co) => co.complete(internal_ctx, ctx).await,
-                    Initiator::Order(od) => od.prompt_confirmation(internal_ctx, ctx).await,
-                },
-                PaymentState::Cancelled => match initiator {
-                    Initiator::Checkout(co) => co.release(ctx).await,
-                    Initiator::Order(od) => od.cancel(internal_ctx, ctx).await,
-                },
-                _ => Ok(()),
-            }
+        PaymentState::Succeeded | PaymentState::Cancelled => {
+            return Err(new_application_error(
+                "PAYMENT_COMPLETED",
+                "the payment has already completed",
+            ))
         }
+        _ => {}
+    }
+
+    let result = ctx.process_payment(payment, &args).await?;
+
+    payment.correlation_id = result.payment_data.correlation_id;
+    payment.data = result.payment_data.data;
+
+    payment.state = result.state;
+
+    ctx.update_payment(payment).await?;
+
+    match payment.state {
+        PaymentState::Succeeded => match initiator {
+            Initiator::Checkout(co) => co.complete(internal_ctx, ctx).await,
+            Initiator::Order(od) => od.prompt_confirmation(internal_ctx, ctx).await,
+        },
+        PaymentState::Cancelled => match initiator {
+            Initiator::Checkout(co) => co.release(ctx).await,
+            Initiator::Order(od) => od.cancel(internal_ctx, ctx).await,
+        },
         _ => Ok(()),
     }
 }
@@ -239,12 +206,12 @@ pub async fn process<'a, C: Context + Send, P: Sync + Send + Serialize + Deseria
 pub async fn cancel<'a, C: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
-    payment: &mut Payment<<C as PaymentProcessor>::Data, <C as PaymentProcessor>::ProcessArgs>,
+    payment: &mut Payment<<C as PaymentProcessor>::Data>,
     initiator: Initiator<'a, P>,
 ) -> Result<(), Error> {
     match &payment.state {
         PaymentState::Cancelled => Ok(()),
-        PaymentState::Idle => {
+        PaymentState::Incomplete => {
             let cancellation_result = ctx.cancel_payment(payment).await?;
             if !cancellation_result.success {
                 return Err(new_application_error(
@@ -261,10 +228,6 @@ pub async fn cancel<'a, C: Context + Send, P: Sync + Send + Serialize + Deserial
                 Initiator::Order(od) => od.cancel(internal_ctx, ctx).await,
             }
         }
-        PaymentState::Processing(_) => Err(new_application_error(
-            "PAYMENT_PROCESSING",
-            "the payment is processing",
-        )),
         PaymentState::Succeeded => Err(new_application_error(
             "PAYMENT_COMPLETED",
             "the payment has already completed",
